import os

from b_aws_testing_framework.credentials import Credentials
from b_aws_testing_framework.tools.cdk_testing.cdk_tool_config import CdkToolConfig
from b_aws_testing_framework.tools.cdk_testing.testing_manager import TestingManager

GLOBAL_PREFIX = os.environ.get('GLOBAL_PREFIX')
CDK_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
MANAGER = TestingManager(Credentials(), CdkToolConfig(CDK_PATH, destroy_before_preparing=False))

if GLOBAL_PREFIX:
    MANAGER.set_global_prefix(f'B{GLOBAL_PREFIX[:10]}')



from typing import Union

from aws_cdk.aws_apigatewayv2 import CfnAuthorizer, CfnApi
from aws_cdk.core import Stack

from b_cfn_custom_userpool_authorizer.config.user_pool_config import UserPoolConfig
from b_cfn_custom_userpool_authorizer.config.user_pool_ssm_config import UserPoolSsmConfig
from b_cfn_custom_userpool_authorizer.user_pool_custom_authorizer_function import AuthorizerFunction


class UserPoolCustomAuthorizer(CfnAuthorizer):
    def __init__(
            self,
            scope: Stack,
            name: str,
            api: CfnApi,
            user_pool_config: Union[UserPoolConfig, UserPoolSsmConfig],
            cache_ttl: int = 60
    ) -> None:
        """
        Constructor.

        :param scope: CloudFormation stack.
        :param name: Name of the custom authorizer e.g. "MyCoolAuthorizer".
        :param api: Parent API for which we are creating the authorizer.
        :param user_pool_config: Configuration of the user pool data.
        :param cache_ttl: The TTL in seconds of cached authorizer results.
            If it equals 0, authorization caching is disabled.
            If it is greater than 0, API Gateway will cache authorizer responses.
            The maximum value is 3600, or 1 hour.
        """
        lambda_function = AuthorizerFunction(
            scope=scope,
            name=f'{name}Function',
            user_pool_config=user_pool_config
        )

        # These environment variables are necessary for a lambda function to create
        # a policy document to allow/deny access. Read more here:
        # https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-lambda-authorizer-output.html
        lambda_function.add_environment('AWS_ACCOUNT', scope.account)
        lambda_function.add_environment('AWS_API_ID', api.ref)

        # Constructed by reading this documentation:
        # https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigatewayv2-authorizer.html
        super().__init__(
            scope=scope,
            id=name,
            name=name,
            api_id=api.ref,
            authorizer_payload_format_version='2.0',
            authorizer_result_ttl_in_seconds=cache_ttl,
            authorizer_type='REQUEST',
            authorizer_uri=(
                f'arn:aws:apigateway:{scope.region}:'
                f'lambda:path/2015-03-31/functions/arn:'
                f'aws:lambda:{scope.region}:{scope.account}:'
                f'function:{lambda_function.function_name}/invocations'
            ),
            identity_source=[
                '$request.header.Authorization'
            ],
        )



from aws_cdk.aws_cognito import *
from aws_cdk.aws_ssm import StringParameter
from aws_cdk.core import Stack
from b_aws_testing_framework.tools.cdk_testing.testing_stack import TestingStack


class UserPoolStack(Stack):
    def __init__(self, scope: Stack):
        super().__init__(
            scope=scope,
            id='UserPoolStack'
        )

        prefix = TestingStack.global_prefix()

        self.pool = UserPool(
            scope=self,
            id='UserPool',
            user_pool_name=f'{prefix}UserPool',
            account_recovery=AccountRecovery.NONE,
            auto_verify=AutoVerifiedAttrs(email=True, phone=False),
            self_sign_up_enabled=False,
            sign_in_aliases=SignInAliases(email=False, phone=False, preferred_username=True, username=True),
            sign_in_case_sensitive=True,
            standard_attributes=StandardAttributes(
                email=StandardAttribute(required=False, mutable=True),
                preferred_username=StandardAttribute(required=True, mutable=True)
            )
        )

        self.client: UserPoolClient = self.pool.add_client(
            id=f'UserPoolClient',
            user_pool_client_name=f'{prefix}UserPoolClient',
            auth_flows=AuthFlow(
                admin_user_password=True,
                user_password=True,
                user_srp=True
            ),
            disable_o_auth=True,
        )

        self.ssm_pool_region = StringParameter(
            scope=self,
            id='UserPoolRegion',
            string_value=self.region,
            parameter_name=f'{prefix}UserPoolRegion'
        )

        self.ssm_pool_id = StringParameter(
            scope=self,
            id='UserPoolId',
            string_value=self.pool.user_pool_id,
            parameter_name=f'{prefix}UserPoolId'
        )

        self.ssm_pool_client_id = StringParameter(
            scope=self,
            id='UserPoolClientId',
            string_value=self.client.user_pool_client_id,
            parameter_name=f'{prefix}UserPoolClientId'
        )



import json
import time

import urllib3
from jose import jwk, jwt
from jose.utils import base64url_decode

from auth_exception import AuthException
from user_pool_resolver import Resolver

KEYS_URL = f'https://cognito-idp.{Resolver.user_pool_region}.amazonaws.com/{Resolver.user_pool_id}/.well-known/jwks.json'
HTTP_MANAGER = urllib3.PoolManager()
KEYS = json.loads(HTTP_MANAGER.request('GET', KEYS_URL).data.decode())['keys']


class TokenVerification:
    """
    Class responsible for access token verification. The inspiration is taken from this example:
    https://github.com/awslabs/aws-support-tools/blob/master/Cognito/decode-verify-jwt/decode-verify-jwt.py
    """
    def __init__(self, access_token: str):
        self.__access_token = access_token

        if not access_token:
            raise AuthException('Access token not provided!')

    def verify(self) -> None:
        """
        Verifies the provided access token. If token is not valid
        an exception is thrown. If no exception is thrown - token is valid.

        :return: No return.
        """
        print(
            f'Verifying access token: {self.__access_token}. '
            f'{Resolver.user_pool_region=}, {Resolver.user_pool_id=}, {Resolver.user_pool_client_id=}.'
        )

        # Get the kid from the headers prior to verification.
        headers = jwt.get_unverified_headers(self.__access_token)
        kid = headers['kid']
        # Search for the kid in the downloaded public keys.
        key_index = -1
        for i in range(len(KEYS)):
            if kid == KEYS[i]['kid']:
                key_index = i
                break
        if key_index == -1:
            raise AuthException('Public key not found in jwks.json!')

        # Construct the public key.
        public_key = jwk.construct(KEYS[key_index])
        # Get the last two sections of the token: message and signature (encoded in base64).
        message, encoded_signature = str(self.__access_token).rsplit('.', 1)
        # Decode the signature.
        decoded_signature = base64url_decode(encoded_signature.encode('utf-8'))

        # Verify the signature.
        if not public_key.verify(message.encode("utf8"), decoded_signature):
            raise AuthException('Signature verification failed!')

        # Since we passed the verification, we can now safely use the unverified claims.
        claims = jwt.get_unverified_claims(self.__access_token)
        print(f'Claims: {claims}.')

        # Additionally we can verify the token expiration.
        if time.time() > claims['exp']:
            raise AuthException('Token is expired!')

        # And the Audience (use claims['client_id'] if verifying an access token). Read more here:
        # https://stackoverflow.com/questions/53148711/why-doesnt-amazon-cognito-return-an-audience-field-in-its-access-tokens
        if (claims.get('aud') or claims.get('client_id')) != Resolver.user_pool_client_id:
            raise AuthException('Token was not issued for this audience')



from typing import Dict, Any


class PolicyDocument:
    """
    Policy document that is constructed according to this documentation:
    https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-lambda-authorizer-output.html
    """
    def __init__(
            self,
            region: str,
            account_id: str,
            api_id: str
    ) -> None:
        self.region = region
        self.account_id = account_id
        self.api_id = api_id

    def create_policy_statement(
            self,
            allow: bool = False
    ) -> Dict[str, Any]:
        return {
            'principalId': 'user',
            'policyDocument': {
                'Version': '2012-10-17',
                'Statement': [
                    {
                        'Action': 'execute-api:Invoke',
                        'Resource': f'arn:aws:execute-api:{self.region}:{self.account_id}:{self.api_id}/*/*',
                        'Effect': 'Allow' if allow else 'Deny'
                    }
                ]
            }
        }



import json

from b_aws_testing_framework.credentials import Credentials
from jose.utils import base64url_decode
from pytest import fixture

from b_cfn_custom_userpool_authorizer_test.integration.infrastructure.main_stack import MainStack


@fixture(scope='function')
def access_token():
    client = Credentials().boto_session.client('cognito-idp')
    user_pool_id = MainStack.get_output(MainStack.USER_POOL_ID_KEY)
    user_pool_client_id = MainStack.get_output(MainStack.USER_POOL_CLIENT_ID_KEY)

    username = 'TestSampleUsername123'

    # A random string to fit the requirements.
    temp_password = ')%2LU5nGNr-TEST'
    new_password = '34#$%ERTre!t3y'

    try:
        # Cleanup before creating user. There might be leftovers from previous runs.
        client.admin_delete_user(
            UserPoolId=user_pool_id,
            Username=username
        )
    except client.exceptions.UserNotFoundException:
        pass

    client.admin_create_user(
        UserPoolId=user_pool_id,
        Username=username,
        TemporaryPassword=temp_password,
        UserAttributes=[
            {'Name': 'preferred_username', 'Value': username},
        ],
    )

    session = client.admin_initiate_auth(
        UserPoolId=user_pool_id,
        ClientId=user_pool_client_id,
        AuthFlow='ADMIN_NO_SRP_AUTH',
        AuthParameters={'USERNAME': username, 'PASSWORD': temp_password},
    )['Session']

    access_token = client.admin_respond_to_auth_challenge(
        UserPoolId=user_pool_id,
        ClientId=user_pool_client_id,
        ChallengeName='NEW_PASSWORD_REQUIRED',
        ChallengeResponses={
            'NEW_PASSWORD': new_password,
            'USERNAME': username,
        },
        Session=session,
    )['AuthenticationResult']['AccessToken']

    # Debug.
    header, payload, signature = access_token.split('.')
    header = json.loads(base64url_decode(header.encode()).decode())
    payload = json.loads(base64url_decode(payload.encode()).decode())
    print(json.dumps(header, indent=4))
    print(json.dumps(payload, indent=4))

    return access_token



from typing import Union

from aws_cdk.aws_iam import PolicyStatement
from aws_cdk.aws_lambda import Function, Code, Runtime, CfnPermission
from aws_cdk.aws_logs import RetentionDays
from aws_cdk.core import Duration, Stack
from b_lambda_layer_common.layer_v2 import LayerV2 as BLayer
from b_lambda_layer_common.package_version import PackageVersion

from b_cfn_custom_userpool_authorizer.config.user_pool_config import UserPoolConfig
from b_cfn_custom_userpool_authorizer.config.user_pool_ssm_config import UserPoolSsmConfig


class AuthorizerFunction(Function):
    def __init__(
            self,
            scope: Stack,
            name: str,
            user_pool_config: Union[UserPoolConfig, UserPoolSsmConfig],
            *args,
            **kwargs
    ) -> None:
        super().__init__(
            scope=scope,
            id=name,
            function_name=name,
            code=self.code(),
            handler='index.handler',
            runtime=Runtime.PYTHON_3_8,
            environment=user_pool_config.to_dict(),
            layers=[
                BLayer(
                    scope=scope,
                    name=f'{name}BCommonLayer',
                    dependencies={
                        'python-jose': PackageVersion.from_string_version('3.3.0')
                    }
                )
            ],
            log_retention=RetentionDays.ONE_MONTH,
            memory_size=128,
            timeout=Duration.seconds(30),
            *args,
            **kwargs
        )

        CfnPermission(
            scope=scope,
            id=f'{name}InvokePermissionForApiGateway',
            action='lambda:InvokeFunction',
            function_name=self.function_name,
            principal='apigateway.amazonaws.com',
        )

        if isinstance(user_pool_config, UserPoolSsmConfig):
            self.add_to_role_policy(
                PolicyStatement(
                    actions=['ssm:GetParameters'],
                    resources=['*']
                )
            )

    @staticmethod
    def code() -> Code:
        from .source import root
        return Code.from_asset(root)



import os

from b_lambda_layer_common.ssm.ssm_parameter import SSMParameter

USER_POOL_REGION_PARAMETER = SSMParameter(os.environ.get('USER_POOL_REGION_SSM_KEY'))
USER_POOL_ID_PARAMETER = SSMParameter(os.environ.get('USER_POOL_ID_SSM_KEY'))
USER_POOL_CLIENT_ID_PARAMETER = SSMParameter(os.environ.get('USER_POOL_CLIENT_ID_SSM_KEY'))


class UserPoolResolver:
    def __init__(self):
        self.user_pool_region = os.environ.get('USER_POOL_REGION') or USER_POOL_REGION_PARAMETER.value
        self.user_pool_id = os.environ.get('USER_POOL_ID') or USER_POOL_ID_PARAMETER.value
        self.user_pool_client_id = os.environ.get('USER_POOL_CLIENT_ID') or USER_POOL_CLIENT_ID_PARAMETER.value


# Expose the instance that is globally cached by a Lambda service.
Resolver = UserPoolResolver()



from aws_cdk.aws_apigatewayv2 import CfnRoute, CfnAuthorizer, CfnApi
from aws_cdk.aws_lambda import Function, Code, Runtime, CfnPermission
from aws_cdk.core import Stack, Duration
from b_aws_testing_framework.tools.cdk_testing.testing_stack import TestingStack
from b_cfn_lambda_integration.lambda_integration import LambdaIntegration


class AuthorizedEndpointStack(Stack):
    def __init__(self, scope: Stack, api: CfnApi, authorizer: CfnAuthorizer):
        super().__init__(
            scope=scope,
            id='AuthorizedEndpointStack'
        )

        prefix = TestingStack.global_prefix()

        self.api_endpoint_function = Function(
            scope=self,
            id='ApiFunction',
            function_name=f'{prefix}ApiFunction',
            code=Code.from_inline(
                'def handler(event, context):\n'
                '    print(event)\n'
                '    return {\n'
                '        "statusCode": 200,\n'
                '        "headers": {},\n'
                '        "body": "Hello World!",\n'
                '        "isBase64Encoded": False'
                '    }'
            ),
            handler='index.handler',
            runtime=Runtime.PYTHON_3_7,
            memory_size=128,
            timeout=Duration.seconds(30),
        )

        CfnPermission(
            scope=self,
            id=f'{prefix}InvokePermission',
            action='lambda:InvokeFunction',
            function_name=self.api_endpoint_function.function_name,
            principal='apigateway.amazonaws.com',
        )

        self.integration = LambdaIntegration(
            scope=self,
            api=api,
            integration_name=f'{prefix}Integration',
            lambda_function=self.api_endpoint_function
        )

        self.route = CfnRoute(
            scope=self,
            id='DummyRoute',
            api_id=api.ref,
            route_key=f'GET /dummy',
            authorization_type='CUSTOM',
            target=f'integrations/{self.integration.ref}',
            authorizer_id=authorizer.ref
        )



import urllib3
from urllib3 import HTTPResponse

from b_cfn_custom_userpool_authorizer_test.integration.infrastructure.main_stack import MainStack
from b_cfn_custom_userpool_authorizer_test.integration.util.urlsafe_json import UrlSafeJson


def test_authorizer_with_invalid_kid_deny(access_token) -> None:
    """
    Tests whether the authorizer denys the request to pass through, if the
    access token is invalid (invalid kid).

    :param access_token: (Fixture) valid access token that will be modified to be invalid.

    :return: No return.
    """
    # Modify the kid so the whole jwt becomes invalid.
    header, payload, signature = access_token.split('.')
    header = UrlSafeJson.decode(header)
    header['kid'] = 'AutxFv/SsJHvxbk2C2w3AOyWp3P6Sg+bP92bUInAAeB='
    header = UrlSafeJson.encode(header)
    access_token = f'{header}.{payload}.{signature}'

    assert __make_call(access_token).status == 403


def test_authorizer_with_invalid_expiration_deny(access_token) -> None:
    """
    Tests whether the authorizer denys the request to pass through, if the
    access token is invalid (invalid expiration time).

    :param access_token: (Fixture) valid access token that will be modified to be invalid.

    :return: No return.
    """
    # Modify the expiration so the whole jwt becomes invalid.
    header, payload, signature = access_token.split('.')
    payload = UrlSafeJson.decode(payload)
    payload['exp'] = 1032393848
    payload = UrlSafeJson.encode(payload)
    access_token = f'{header}.{payload}.{signature}'

    assert __make_call(access_token).status == 403


def test_authorizer_with_no_access_token() -> None:
    """
    Tests whether the authorizer denys the request to pass through, if
    there is no access token.

    :return: No return.
    """
    endpoint = MainStack.get_output(MainStack.API_ENDPOINT_KEY)

    http = urllib3.PoolManager()

    response = http.request(
        method='GET',
        url=endpoint,
        headers={},
    )

    assert response.status == 401


def __make_call(access_token) -> HTTPResponse:
    endpoint = MainStack.get_output(MainStack.API_ENDPOINT_KEY)

    http = urllib3.PoolManager()

    return http.request(
        method='GET',
        url=endpoint,
        headers={
            'Authorization': access_token
        },
    )



from typing import Optional, Dict


class UserPoolConfig:
    def __init__(
            self,
            user_pool_id: Optional[str] = None,
            user_pool_region: Optional[str] = None,
            user_pool_client_id: Optional[str] = None,
    ) -> None:
        self.user_pool_id = user_pool_id
        self.user_pool_region = user_pool_region
        self.user_pool_client_id = user_pool_client_id

    def to_dict(self) -> Dict[str, str]:
        return {
            'USER_POOL_ID': self.user_pool_id,
            'USER_POOL_REGION': self.user_pool_region,
            'USER_POOL_CLIENT_ID': self.user_pool_client_id,
        }



from aws_cdk.aws_apigatewayv2 import CfnApi, CfnStage
from aws_cdk.core import Construct
from b_aws_testing_framework.tools.cdk_testing.testing_stack import TestingStack

from b_cfn_custom_userpool_authorizer.config.user_pool_ssm_config import UserPoolSsmConfig
from b_cfn_custom_userpool_authorizer.user_pool_custom_authorizer import UserPoolCustomAuthorizer
from b_cfn_custom_userpool_authorizer_test.integration.infrastructure.authorized_endpoint_stack import AuthorizedEndpointStack
from b_cfn_custom_userpool_authorizer_test.integration.infrastructure.user_pool_stack import UserPoolStack


class MainStack(TestingStack):
    API_URL_KEY = 'ApiUrl'
    API_ENDPOINT_KEY = 'ApiEndpoint'
    USER_POOL_ID_KEY = 'UserPoolId'
    USER_POOL_CLIENT_ID_KEY = 'UserPoolClientId'

    def __init__(self, scope: Construct) -> None:
        super().__init__(scope=scope)

        prefix = TestingStack.global_prefix()

        self.user_pool_stack = UserPoolStack(self)

        self.api = CfnApi(
            scope=self,
            id='Api',
            name=f'{prefix}Api',
            description='Sample description.',
            protocol_type='HTTP',
            cors_configuration=CfnApi.CorsProperty(
                allow_methods=['GET', 'PUT', 'POST', 'OPTIONS', 'DELETE'],
                allow_origins=['*'],
                allow_headers=[
                    'Content-Type',
                    'Authorization'
                ],
                max_age=300
            )
        )

        self.authorizer = UserPoolCustomAuthorizer(
            scope=self,
            name=f'{prefix}UserPoolCustomAuthorizer',
            api=self.api,
            # Dynamically resolve.
            user_pool_config=UserPoolSsmConfig(
                user_pool_id_ssm_key=self.user_pool_stack.ssm_pool_id.parameter_name,
                user_pool_client_id_ssm_key=self.user_pool_stack.ssm_pool_client_id.parameter_name,
                user_pool_region_ssm_key=self.user_pool_stack.ssm_pool_region.parameter_name,
            )
        )

        self.stage: CfnStage = CfnStage(
            scope=self,
            id='Stage',
            stage_name='test',
            api_id=self.api.ref,
            auto_deploy=True,
        )

        self.endpoint_stack = AuthorizedEndpointStack(self, self.api, self.authorizer)

        self.add_output(self.API_URL_KEY, value=self.api.attr_api_endpoint)
        self.add_output(self.API_ENDPOINT_KEY, value=f'{self.api.attr_api_endpoint}/{self.stage.stage_name}/dummy')
        self.add_output(self.USER_POOL_ID_KEY, value=self.user_pool_stack.pool.user_pool_id)
        self.add_output(self.USER_POOL_CLIENT_ID_KEY, value=self.user_pool_stack.client.user_pool_client_id)



import json
import os

from auth_exception import AuthException
from policy_document import PolicyDocument
from token_verification import TokenVerification


def handler(event, context):
    print(f'Received event:\n{json.dumps(event)}')

    document = PolicyDocument(
        region=os.environ['AWS_REGION'],
        account_id=os.environ['AWS_ACCOUNT'],
        api_id=os.environ['AWS_API_ID']
    )

    # Verify the authorization token.
    try:
        TokenVerification(event.get('headers', {}).get('authorization')).verify()
        # Authorization was successful. Return "Allow".
        return document.create_policy_statement(allow=True)
    except AuthException as ex:
        # Log the error.
        print(ex)
        # Authorization has failed. Return "Deny".
        return document.create_policy_statement(allow=False)



# Import all the fixtures.
# noinspection PyUnresolvedReferences
from b_cfn_custom_userpool_authorizer_test.integration.fixtures import *

from b_cfn_custom_userpool_authorizer_test.integration.infra_create import inf_create
from b_cfn_custom_userpool_authorizer_test.integration.infra_destroy import inf_destroy


def pytest_sessionstart(session):
    inf_create()


def pytest_sessionfinish(session, exitstatus):
    inf_destroy()



import json
from typing import Any, Dict

from jose.utils import base64url_decode, base64url_encode


class UrlSafeJson:
    @staticmethod
    def decode(data: str) -> Dict[str, Any]:
        data = base64url_decode(data.encode())
        return json.loads(data)

    @staticmethod
    def encode(data: Dict[str, Any]) -> str:
        data = json.dumps(data).replace(' ', '').replace('/', '\\/').encode()
        return base64url_encode(data).decode()



from setuptools import setup, find_packages

with open('README.md') as readme_file:
    README = readme_file.read()

with open('HISTORY.md') as history_file:
    HISTORY = history_file.read()

with open('VERSION') as file:
    VERSION = file.read()
    VERSION = ''.join(VERSION.split())

setup(
    name='b_cfn_custom_userpool_authorizer',
    version=VERSION,
    license='Apache License 2.0',
    packages=find_packages(exclude=[
        # Exclude virtual environment.
        'venv',
        # Exclude test source files.
        'b_cfn_custom_userpool_authorizer_test'
    ]),
    description=(
        'Package that enables deployment of AWS CDK stacks in parallel.'
    ),
    long_description=README + '\n\n' + HISTORY,
    long_description_content_type='text/markdown',
    include_package_data=True,
    install_requires=[
        'python-jose>=3.3.0',
        'b-continuous-subprocess>=0.3.2,<1.0.0',
        'b-lambda-layer-common>=1.20.0,<2.0.0',
        'b-aws-cdk-parallel>=1.3.0,<2.0.0',
        'b-cfn-lambda-integration>=0.0.8,<1.0.0',
        'aws_cdk.aws_apigatewayv2>=1.90.0',
        'aws_cdk.aws_lambda>=1.90.0',
        'aws-cdk.assets>=1.90.0',
        'aws-cdk.aws-ec2>=1.90.0',
        'aws-cdk.core>=1.90.0',
        'aws-cdk.custom-resources>=1.90.0',
        'aws-cdk.cloud-assembly-schema>=1.90.0',
        'aws-cdk.region-info>=1.90.0',
        'aws-cdk.aws-cognito>=1.90.0',
    ],
    keywords='AWS Cognito UserPool Authorizer',
    url='https://github.com/biomapas/B.CfnCustomUserPoolAuthorizer.git',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
)



from typing import Optional, Dict


class UserPoolSsmConfig:
    def __init__(
            self,
            user_pool_id_ssm_key: Optional[str] = None,
            user_pool_region_ssm_key: Optional[str] = None,
            user_pool_client_id_ssm_key: Optional[str] = None
    ) -> None:
        self.user_pool_id_ssm_key = user_pool_id_ssm_key
        self.user_pool_region_ssm_key = user_pool_region_ssm_key
        self.user_pool_client_id_ssm_key = user_pool_client_id_ssm_key

    def to_dict(self) -> Dict[str, str]:
        return {
            'USER_POOL_ID_SSM_KEY': self.user_pool_id_ssm_key,
            'USER_POOL_REGION_SSM_KEY': self.user_pool_region_ssm_key,
            'USER_POOL_CLIENT_ID_SSM_KEY': self.user_pool_client_id_ssm_key,
        }



import urllib3
from urllib3 import HTTPResponse

from b_cfn_custom_userpool_authorizer_test.integration.infrastructure.main_stack import MainStack


def test_authorizer_allow(access_token: str) -> None:
    """
    Tests whether the authorizer allows the request to pass through, if the
    access token is valid.

    :param access_token: (Fixture) valid access token.

    :return: No return.
    """
    endpoint = MainStack.get_output(MainStack.API_ENDPOINT_KEY)

    http = urllib3.PoolManager()

    response: HTTPResponse = http.request(
        method='GET',
        url=endpoint,
        headers={
            'Authorization': access_token
        },
    )

    assert response.status == 200

    data = response.data
    data = data.decode()
    # Response from a dummy lambda function defined in the infrastructure main stack.
    assert data == 'Hello World!'



import os

from b_cfn_custom_userpool_authorizer_test.integration.manager import MANAGER

DO_NOT_DESTROY_INFRASTRUCTURE = int(os.environ.get('DO_NOT_DESTROY_INFRASTRUCTURE', 0))


def inf_destroy():
    if DO_NOT_DESTROY_INFRASTRUCTURE == 1:
        return

    MANAGER.destroy_infrastructure()


if __name__ == '__main__':
    inf_destroy()
